/* SPDX-License-Identifier: GPL-2.0 */
/*
 * This header provides constants clk index RK808 pmic clkout
 */
#ifndef _CLK_ROCKCHIP_RK808
#define _CLK_ROCKCHIP_RK808

/* CLOCKOUT index */
#define RK808_CLKOUT0		0
#define RK808_CLKOUT1		1

#endif
