/* SPDX-License-Identifier: GPL-2.0+ WITH Linux-syscall-note */
/*
 * Copyright 1999-2000 Jeremy Fitzhardinge <jeremy@goop.org>
 *
 * This file is part of the Linux kernel and is made available under
 * the terms of the GNU General Public License, version 2, or at your
 * option, any later version, incorporated herein by reference.
 */

#ifndef _UAPI_LINUX_AUTO_FS4_H
#define _UAPI_LINUX_AUTO_FS4_H

#include <linux/auto_fs.h>

#endif /* _UAPI_LINUX_AUTO_FS4_H */
